package strsets; 

import static java.util.Arrays.binarySearch;
import static java.util.Arrays.sort;

/**
 * Una implementazione di SortedStrSet.
 * La classe e' anche definita come estensione di KeywordSet.
 */
public class SortedKeywordSet extends KeywordSet implements SortedStrSet {

    /**
     * Costruisce un insieme di keywords copiandole dall'array kA.
     * Le keywords sono poi ordinate in senso ascendente rispetto
     * all'ordinamento lessicografico
     * 
     * @param ka    array di keywords
     * @return      l'insieme ordinato contenente una copia delle kywords in kA
     */
    public SortedKeywordSet(String[] kA) {
        super(kA);  // crea l'insieme copiando l'array
        sort(keywords);  // ordina l'array in senso ascendente rispetto
                                // all'ordine lessicografico prodotto dal metodo
                                // compareTo() di String
    }                             
                                  
    /**
     * Verifica se una stringa e' contenuta nell'insieme
     * 
     * Sovrascrive il metodo di KeywordSet.
     * Dato che l'array e' ordinato puo' usare (migliorando l'efficienza)
     * la ricerca binaria definita nella classe Array.
     * 
     * @param s     la stringa
     * @return      true se la stringa e' contenuta, false altrimenti
     */
    public boolean contains(String s) {
        // usa la ricerca binaria per cercare la stringa
        return (binarySearch(keywords, s) >= 0);
    }
    
    /**
     * Ritorna la k-esima stringa dell'insieme ordinato.
     * (implementazione di un metodo dell'interfaccia)
     * 
     * @param k     la posizione della stringa
     * @return      La stringa nella k-esima posizione.
     *              Se k < 0 oppure k > size - 1 ritorna null.
     */
    public String getKth(int k) {
        if (k < 0 || k > keywords.length - 1)
            return null;
        return keywords[k];
    }
}